/*
 * ControllerBean.java
 *
 * Created on January 20, 2007, 7:56 PM
 *
 */

package info.heinzelnisse.he;

import java.awt.event.KeyEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.logging.Logger;

/**
 * Controller: Singleton, since there can be only one
 *
 * @author Heiko Klein
 */
public class ControllerBean implements Serializable, HistoryChangeListener {
    static private Logger logger = Logger.getLogger(ControllerBean.class.getName());
    static private ControllerBean instance = null;
    
    public static final String NORWEGIAN = "norwegian";
    public static final String GERMAN = "german";
    public static final String SEARCH_STRING = "searchString";
    public static final String INDEXWORD = "indexword";
    public static final String TRANSLATION = "translation";
    public static final String HISTORY_PREVIOUS = "historyPrevious";
    public static final String HISTORY_NEXT = "historyNext";
    
    private final PropertyChangeSupport pcs = new PropertyChangeSupport(this);
    private History<TranslationGroup> history;
    private List<SearchKeyEventListener> searchKeyEventListeners = new ArrayList<SearchKeyEventListener>();
    private String searchString = "Heinzelnisse";
    private boolean norwegian = true;
    private boolean german = true;
    private boolean cutnpaste = false;
    List<String> sortedItems = null;
    HeinzelTable heinzelTable = null;
    
    private TranslationGroup translationGroup = null;
    
    private String indexWord = null;

    static synchronized public ControllerBean getInstance() {
        if (instance == null) {
            instance = new ControllerBean();
        }
        return instance;
    }
    
    /** Creates a new instance of ControllerBean */
    private ControllerBean() {
        heinzelTable = new HeinzelTable();
        history = new History<TranslationGroup>(100,700);
        history.addHistoryChangeListener(this);
        Thread htReader = new Thread(heinzelTable);
        htReader.setPriority(Thread.MIN_PRIORITY);
        htReader.start();
    }
    
    
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        this.pcs.addPropertyChangeListener(listener);
    }
    
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        this.pcs.removePropertyChangeListener(listener);
    }
    
    public void addSearchKeyEventListener(SearchKeyEventListener listener) {
        searchKeyEventListeners.add(listener);
    }
    
    
    public String getSearchString() {
        return searchString;
    }
    
    public void setSearchString(String searchString) {
        String old = this.searchString;
        logger.info("got string "+ searchString);
        this.searchString = searchString;
        this.pcs.firePropertyChange(SEARCH_STRING, old, searchString);
    }
    
    /**
     * get the closest index of the sorted item list corresponding to the searchString
     */
    public int getSearchStringIdx() {
        return getSearchStringIdx(getSearchString());
    }
    public int getSearchStringIdx(String otherSearchString) {
        int idx = Collections.binarySearch(getSortedItems(), otherSearchString, getHeinzelTable().getDeNoCollator());
        if (idx < 0) {
            idx *= -1;
            idx--;
        }
        return idx;
    }
    
    public void setIndexWord(Object indexWord) {
        logger.info("got indexWord "+indexWord);
        if (indexWord == null)
            return;
        String old = this.indexWord;
        this.indexWord = indexWord.toString();
        this.pcs.firePropertyChange(INDEXWORD, old, this.indexWord);
        setTranslationsByIndexWord();
    }
    
    public boolean isNorwegian() {
        return norwegian;
    }
    
    public void setNorwegian(boolean norwegian) {
        if (this.norwegian != norwegian) {
            boolean old = this.norwegian;
            this.norwegian = norwegian;
            sortedItems = null; // no longer up to date
            this.pcs.firePropertyChange(NORWEGIAN, old, norwegian);
        }
    }
    
    public boolean isGerman() {
        return german;
    }
    
    public void setGerman(boolean german) {
        if (this.german != german) {
            boolean old = this.german;
            this.german = german;
            sortedItems = null; // no longer up to date
            this.pcs.firePropertyChange(GERMAN, old, german);
        }
    }
    
    public HeinzelTable getHeinzelTable() {
        return heinzelTable;
    }
    
    public List<String> getSortedItems() {
        if (sortedItems == null) {
            sortedItems = heinzelTable.getSortedItems(isNorwegian(), isGerman());
            if (sortedItems == null) {
                return new ArrayList<String>();
            }
        }
        return sortedItems;
    }
    
    public void setTranslationsByWildcard(String wildCardString) {
        TranslationGroup transGroup = new TranslationGroup();
        List<String> translations = new ArrayList<String>(heinzelTable.searchTranslations(wildCardString));
        logger.info("found "+translations.size()+" trans for "+wildCardString);
        transGroup.setMixedTranslation(Translation.stringCollectionToTranslationCollection(translations, Translation.DE));
        setTranslationGroup(transGroup);
    }
    
    public void setTranslationsByExactWord(String exactWord) {
        TranslationGroup transGroup = new TranslationGroup();
        if (isGerman()) {
            Collection<String> gt = heinzelTable.getGermanTranslationItems().get(exactWord);
            transGroup.setGermanTranslation(Translation.stringCollectionToTranslationCollection(gt, Translation.DE));
        }
        if (isNorwegian()) {
            Collection<String> nt = heinzelTable.getNorwegianTranslationItems().get(exactWord);
            transGroup.setNorwegianTranslation(Translation.stringCollectionToTranslationCollection(nt, Translation.NO));
        }
        setTranslationGroup(transGroup);
    }
    
    public void setTranslationsByAdvancedSearch(String word, boolean caseSensitive, boolean substring, boolean wordNO, boolean articleNO, boolean optionsNO, boolean otherNO, boolean wordDE, boolean articleDE, boolean optionsDE, boolean otherDE, boolean category) {
        Collection<String> translations = getHeinzelTable().searchFulltextTranslations(word, caseSensitive, substring, wordNO, articleNO, optionsNO, otherNO, wordDE, articleDE, optionsDE, otherDE, category);
        TranslationGroup tg = new TranslationGroup();
        tg.setNorwegianTranslation(Translation.stringCollectionToTranslationCollection(translations, Translation.DE));
        setTranslationGroup(tg);
    }
    
    public void setTranslationsByIndexWord() {
        TranslationGroup transGroup = new TranslationGroup();
        // scroll through the indices
        int index = getSearchStringIdx(indexWord);
        int end = (index + 30 < sortedItems.size()) ? (index + 30) : sortedItems.size();
        for (int i = index; i < end; ++i) { // add some translations to the end
            String currentWord = sortedItems.get(i);
            if (isGerman()) {
                Collection<String> gt = heinzelTable.getGermanTranslationItems().get(currentWord);
                Collection<Translation> tgt = Translation.stringCollectionToTranslationCollection(gt, Translation.DE);
                if (tgt != null)
                    for (Translation t : tgt)
                        t.setIsExact(i == index);
                transGroup.addMixedTranslation(tgt);
            }
            if (isNorwegian()) {
                Collection<String> nt = heinzelTable.getNorwegianTranslationItems().get(currentWord);
                Collection<Translation> tnt = Translation.stringCollectionToTranslationCollection(nt, Translation.NO);
                if (tnt != null)
                    for (Translation t : tnt)
                        t.setIsExact(i == index);
                transGroup.addMixedTranslation(tnt);
            }
        }
        setTranslationGroup(transGroup);
    }
    
    public TranslationGroup getTranslationGroup() {
        return translationGroup;
    }
    protected void setTranslationGroup(TranslationGroup translationGroup) {
        TranslationGroup old = this.translationGroup;
        this.translationGroup = translationGroup;
        pcs.firePropertyChange(TRANSLATION, old, translationGroup);
        if (translationGroup.size() <= HeinzelTable.MAX_HISTORY_ITEMS) {
            logger.info("adding translationGroup to history");
            history.addItem(translationGroup);
        }
    }
    
    public void fireSearchKeyReleased(KeyEvent evt) {
        for (SearchKeyEventListener li : searchKeyEventListeners) {
            li.searchKeyReleased(evt);
        }
    }

    @Override
    public void historyChanged() {
        this.pcs.firePropertyChange(HISTORY_NEXT, !history.hasNext(), history.hasNext());
        this.pcs.firePropertyChange(HISTORY_PREVIOUS, !history.hasPrevious(), history.hasPrevious());
    }

    void getHistoryNext() {
        TranslationGroup next = history.goForward();
        this.setTranslationGroup(next);
    }

    void getHistoryPrevious() {
        TranslationGroup previous = history.goBack();
        this.setTranslationGroup(previous);
    }

    /**
     * @return the cutnpaste
     */
    public boolean isCutnpaste() {
        return cutnpaste;
    }

    /**
     * @param cutnpaste the cutnpaste to set
     */
    public void setCutnpaste(boolean cutnpaste) {
        this.cutnpaste = cutnpaste;
    }
    
    
}
